import { combineConfig, Facet } from "@codemirror/state";
import { Decoration, MatchDecorator, EditorView, ViewPlugin, WidgetType, } from "@codemirror/view";
export const highlightPreviewReplacementFacet = Facet.define({
    combine(value) {
        return combineConfig(value, {
            from: "",
            to: "",
        });
    },
});
export const highlightPreviewReplacement = ViewPlugin.fromClass(class {
    placeholders;
    constructor(view) {
        const config = view.state.facet(highlightPreviewReplacementFacet);
        if (!config.from) {
            this.placeholders = Decoration.none;
            return;
        }
        this.placeholders = getPlaceholderMatcher(config.from, config.to).createDeco(view);
    }
    update(update) {
        const config = update.state.facet(highlightPreviewReplacementFacet);
        if (!config.from) {
            this.placeholders = Decoration.none;
            return;
        }
        this.placeholders = getPlaceholderMatcher(config.from, config.to).updateDeco(update, this.placeholders);
    }
}, {
    decorations: (instance) => instance.placeholders,
    provide: (plugin) => EditorView.atomicRanges.of((view) => {
        return view.plugin(plugin)?.placeholders || Decoration.none;
    }),
});
class PlaceholderWidget extends WidgetType {
    name;
    to;
    constructor(name, to) {
        super();
        this.name = name;
        this.to = to;
    }
    eq(other) {
        return this.name === other.name && this.to === other.to;
    }
    toDOM() {
        const elt = document.createElement("span");
        if (this.to) {
            const fromElt = document.createElement("span");
            fromElt.className = "cm-preview-replace-from";
            fromElt.style.cssText = `
      text-decoration: line-through;
      color: #7f1d1d;
      background: #fca5a5;`;
            fromElt.textContent = this.name;
            const toElt = document.createElement("span");
            toElt.className = "cm-preview-replace-to";
            toElt.style.cssText = `
      color: green;
      color: #14532d;
      background: #ecfccb;`;
            toElt.textContent = this.to;
            elt.appendChild(fromElt);
            elt.appendChild(toElt);
        }
        else {
            const fromElt = document.createElement("span");
            fromElt.className = "cm-preview-replace-match";
            fromElt.style.cssText = `
      background: yellow;`;
            fromElt.textContent = this.name;
            elt.appendChild(fromElt);
        }
        return elt;
    }
    ignoreEvent() {
        return false;
    }
}
/**
 * https://github.com/toss/es-toolkit/blob/main/src/string/escapeRegExp.ts
 */
function escapeRegExp(str) {
    return str.replace(/[\\^$.*+?()[\]{}|]/g, "\\$&");
}
function getPlaceholderMatcher(fromstr, to) {
    const regexp = typeof fromstr === "string"
        ? new RegExp(`(${escapeRegExp(fromstr)})`, "g")
        : fromstr;
    return new MatchDecorator({
        regexp,
        decoration: (match) => {
            return Decoration.replace({
                widget: new PlaceholderWidget(String(match[1]), to),
            });
        },
    });
}
//# sourceMappingURL=index.js.map